#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Inspire by Conbagui tutorial found here:https://www.deviantart.com/conbagui/art/Create-Honeycomb-Text-effect-with-Gimp-2-10-793248185
# author: Pocholo
# date: 4/9/22

# Comments directed to http://gimpchat.com

# Installation:
# This script should be placed in the user plugin folder

# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.

from gimpfu import *
import gettext
import os, sys,time 
gettext.install("gimp20", gimp.locale_directory, unicode=True)

def pm_create_a_honey_comb_text(customtext, font, mergeLayers):
		
    #Initiates the temporary state
    pdb.gimp_context_push()
    pdb.gimp_context_set_defaults()
    
    #Create image
    img = pdb.gimp_image_new(1, 1, RGB_IMAGE)
    
    #Create the text
    size = 300
    pdb.gimp_context_set_foreground((44, 40, 25))
    textLayer = pdb.gimp_text_fontname(img, None, 0, 0, customtext, 164, 
                                        TRUE, size, PIXELS, font)
    pdb.gimp_text_layer_set_justification(textLayer, 2)
    pdb.gimp_text_layer_set_letter_spacing(textLayer, 0)
   
    #Resize text
    img.resize(textLayer.width, textLayer.height, 0, 0)
        
    #Rename the text layer and move up
    pdb.gimp_item_set_name(textLayer, "Text")
    textLayer = pdb.gimp_image_get_layer_by_name(img, "Text")
    x = 0
    y = 50
    textLayer.set_offsets(x, y)
    pdb.gimp_layer_resize_to_image_size(textLayer)

    gimp.Display(img)	
 
    #Create wall background
    wallLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Wall background", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_insert_layer(img, wallLayer, None, 1)
    pdb.gimp_drawable_fill(wallLayer, FILL_WHITE)
    pdb.gimp_context_set_foreground((255, 255, 255))
    pdb.gimp_context_set_background((115, 86, 43))
    pdb.gimp_edit_blend(wallLayer, 0, LAYER_MODE_NORMAL, 7, 100, 0, 0, FALSE, TRUE, 3, 0.20, TRUE, 800, 450, 1600, 450)
    pdb.plug_in_gauss(img, wallLayer, 400, 400, 0)
    pdb.plug_in_gauss(img, wallLayer, 400, 400, 0)

    #Create the floor
    floor_x = 0
    floor_y = (img.height - wallLayer.height) / 2 
    floor_width = img.width
    floor_height = img.height / 2 
    floorLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Floor", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_insert_layer(img, floorLayer, None, 1)
    pdb.gimp_drawable_fill(floorLayer, FILL_TRANSPARENT)
    pdb.gimp_image_select_rectangle(img, 2, floor_x, floor_y, floor_width, floor_height -10)
    pdb.gimp_context_set_pattern("Pine?")
    pdb.gimp_bucket_fill(floorLayer, BUCKET_FILL_PATTERN, LAYER_MODE_NORMAL, 100, 255,  FALSE, 800, 235)
    pdb.gimp_selection_none(img)
    floorLayer = pdb.gimp_item_transform_flip_simple(floorLayer, ORIENTATION_VERTICAL, TRUE, 0)
    
    #Create the wall bottom board
    board_x = 0
    board_y = (img.height / 2)
    board_width = img.width
    board_height = img.height / 2 
    boardLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, " Base molding", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_insert_layer(img, boardLayer, None, 1)
    pdb.gimp_drawable_fill(boardLayer, FILL_TRANSPARENT)
    pdb.gimp_image_select_rectangle(img, 2, board_x, board_y, board_width, 40)
    pdb.gimp_context_set_foreground((82, 50, 28))
    pdb.gimp_edit_fill(boardLayer, FILL_FOREGROUND)
    pdb.gimp_selection_none(img)
    pdb.script_fu_drop_shadow(img, boardLayer, -2, 2, 10, "Black", 100, FALSE)
    
    #Merge the floor and wall
    pdb.gimp_item_set_visible(textLayer, FALSE)
    wallLayer = pdb.gimp_image_merge_visible_layers(img, EXPAND_AS_NECESSARY)
    pdb.gimp_item_set_name(wallLayer, "Background")
    pdb.gimp_item_set_visible(textLayer, TRUE)
    pdb.gimp_layer_resize_to_image_size(wallLayer)
    
    #Create honey comb pattern
    cpw = 400
    cph = 232
    comblayer = pdb.gimp_layer_new(img, cpw, cph, RGBA_IMAGE, "Honey comb", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_insert_layer(img, comblayer, None, 0)
    pdb.gimp_context_set_background((127, 127, 127))
    pdb.gimp_drawable_fill(comblayer, FILL_BACKGROUND)
    cpx = (img.width - comblayer.width) / 2
    cpy = (img.height - comblayer.height) / 2
    comblayer.set_offsets(cpx, cpy)
    
    layer = pdb.gimp_layer_new(img, cpw, cph, RGBA_IMAGE, "Layer", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_insert_layer(img, layer, None, 1)
    pdb.gimp_context_set_foreground((213, 213, 213))
    pdb.gimp_drawable_fill(layer, FILL_FOREGROUND)
    cpx = (img.width - layer.width) / 2
    cpy = (img.height - layer.height) / 2
    layer.set_offsets(cpx, cpy)

    pdb.plug_in_mosaic(img, layer, 
							  100,    #tile_size
							  1.0,    #tile_height
								9,    #tile_spacing 
								1,    #tile_neatness 
								1,    #tile_allow_split 
							   90,    #light_dir 
								0,    #color_variation 
								1,    #antialiasing 
								0,    #color_averaging 
								1,    #tile_type 
								0,    #tile_surface 
								0)    #grout_color
    
    pdb.gimp_layer_scale(layer, 876, 525, TRUE)
    layer = pdb.gimp_item_transform_translate(layer, -62, 38)
    pdb.gimp_image_select_item(img, 2, comblayer)
    x0,y0 = pdb.gimp_drawable_offsets(layer)
    non_empty, x1, y1, x2, y2 = pdb.gimp_selection_bounds(img)
    pdb.gimp_layer_resize(layer,x2-x1,y2-y1,x0-x1,y0-y1)
    pdb.gimp_selection_none(img)
    pdb.gimp_image_select_item(img, 2, layer)
    pdb.gimp_drawable_edit_clear(comblayer)
    pdb.gimp_selection_none(img)
    
    
    combDsha = pdb.gimp_layer_new_from_drawable(comblayer, img)
    pdb.gimp_image_insert_layer(img, combDsha, None, 1)
    pdb.gimp_layer_set_mode(combDsha, LAYER_MODE_MULTIPLY_LEGACY)
    pdb.gimp_layer_set_opacity(combDsha, 100)
    pdb.gimp_item_set_name(combDsha, "Honey comb-dropshadow")
    pdb.gimp_image_select_item(img, 2, combDsha)
    pdb.gimp_context_set_foreground("#000000")
    pdb.gimp_edit_bucket_fill(combDsha,  
                        BUCKET_FILL_FG,  #fill_mode
                     LAYER_MODE_NORMAL,  #paint_mode
                                   100,  #opacity
                                   255,  #threshold
                                 FALSE,  #sample_merged
                                     0,  #x
                                     0)  #y
    pdb.gimp_selection_none(img) 
    pdb.plug_in_gauss(img, combDsha, 70, 70, 1)
    pdb.gimp_levels(combDsha, 
                            4,   #channel - histogram-alpha
                            0,   #low_input 
                          135,   #high_input 
                         2.50,   #gamma 
                            0,  #low_output
                          255)  #high_output 
                                    
    pdb.plug_in_gauss(img, combDsha, 70, 70, 1)
    pdb.gimp_levels(combDsha, 
                            4,   #channel - histogram-alpha
                            0,   #low_input 
                          200,   #high_input 
                         1.12,   #gamma 
                            0,  #low_output
                          255)  #high_output 
    
    
    #Inner shadow
    color = ("#ffffff")
    opacity = 75.0
    contour = 0
    noise = 0.0
    mode =  4       #LAYER_MODE_SCREEN_LEGACY
    source = 1
    choke = 0
    size = 20
    offsetangle = 90.0
    offsetdist = 14
    merge = TRUE
    pdb.python_layerfx_inner_shadow(img, comblayer, color, opacity, contour, noise, mode, source, choke, size, offsetangle, offsetdist, merge)    
    comblayer = pdb.gimp_image_get_layer_by_name(img, "Honey comb")
    
    #Bevel and Emboss
    style = 1                      #Inner bevel
    depth = 65
    direction = 0                  #Up
    size = 14
    soften = 7
    angle = 90.0
    altitude = 55.0
    glosscontour = 0                #Linear
    highlightcolor = ("#ffffff")
    highlightmode = 5               #LAYER_MODE_OVERLAY_LEGACY
    highlightopacity = 75.0
    shadowcolor = ("#000000")
    shadowmode = 3                  #LAYER_MODE_MULTIPLY_LEGACY
    shadowopacity = 30.0
    surfacecontour = 6              #Half-round
    use_texture = FALSE
    pattern = "3D Green"
    scale = 100.0
    tex_depth = 100.0
    invert = FALSE
    merge = TRUE
    pdb.python_layerfx_bevel_emboss(img, comblayer, 
                                    style, depth, direction, size, soften, angle, altitude, glosscontour, highlightcolor, highlightmode, 
                                    highlightopacity, shadowcolor, shadowmode, 
                                    shadowopacity, surfacecontour, use_texture, pattern, scale, tex_depth, invert, merge)
    
    comblayer = pdb.gimp_image_get_layer_by_name(img, "Honey comb")
    
    #Merge the pattern layers and copy to clipboard
    shadowLayer1 = pdb.gimp_image_get_layer_by_name(img, "Honey comb-dropshadow")
    pdb.gimp_image_select_item(img, 2, comblayer)
    x0,y0 = pdb.gimp_drawable_offsets(shadowLayer1)
    non_empty, x1, y1, x2, y2 = pdb.gimp_selection_bounds(img)
    pdb.gimp_layer_resize(shadowLayer1,x2-x1,y2-y1,x0-x1,y0-y1)
    pdb.gimp_selection_none(img)
    layer = pdb.gimp_image_merge_down(img, shadowLayer1, CLIP_TO_IMAGE)
    layer = pdb.gimp_image_get_layer_by_name(img, "Layer")
    layer = pdb.gimp_image_merge_down(img, comblayer, CLIP_TO_IMAGE)
    layer = pdb.gimp_image_get_layer_by_name(img, "Layer")
    pdb.plug_in_make_seamless(img, layer)
    pdb.gimp_layer_scale(layer, 200, 116, TRUE)
    non_empty = pdb.gimp_edit_copy(layer)
    pdb.gimp_image_remove_layer(img, layer)
    
    #Text drop shadow
    color = "#000000"
    opacity = 100.0
    contour = 0        #Linear
    noise = 0.0
    mode = 3           #LAYER_MODE_MULTIPLY_LEGACY
    spread = 0
    size = 9
    offsetangle = 90
    offsetdist = 6
    knockout = TRUE
    merge = FALSE
    pdb.python_layerfx_drop_shadow(img, textLayer, color, opacity, contour, noise, mode, spread, size, offsetangle, offsetdist, knockout, merge)
    comblayer = pdb.gimp_image_get_layer_by_name(img, "Honey comb-dropshadow")
    
    #Text Inner shadow
    color = ("#a1390b")
    opacity = 100.0
    contour = 0         #Linear
    noise = 0.0
    mode =  5           #LAYER_MODE_OVERLAY_LEGACY
    source = 1
    choke = 0
    size = 21
    offsetangle = 120.0
    offsetdist = 21
    merge = FALSE
    pdb.python_layerfx_inner_shadow(img, textLayer, color, opacity, contour, noise, mode, source, choke, size, offsetangle, offsetdist, merge)
    inshadlayer = pdb.gimp_image_get_layer_by_name(img, "Text-innershadow")
    
    #Text inner glow
    color = "#2d0202"
    opacity = 100.0
    contour = 10        #Rounded step
    noise = 0.0
    mode =5             #LAYER_MODE_OVERLAY_LEGACY
    source = 1
    choke = 10.0
    size = 17
    merge = FALSE
    pdb.python_layerfx_inner_glow(img, textLayer, color, opacity, contour, noise, mode, source, choke, size, merge)
    
    #Text bevel and emboss
    style = 1                      #Inner bevel
    depth = 65
    direction = 0                  #Up
    size = 12
    soften = 0
    angle = 90.0
    altitude = 40.0
    glosscontour = 0                #Linear
    highlightcolor = ("#ffffff")
    highlightmode = 4               #LAYER_MODE_SCREEN_LEGACY
    highlightopacity = 100.0
    shadowcolor = ("#c4af7b")
    shadowmode = 16                  #LAYER_MODE_DODGE_LEGACY
    shadowopacity = 100.0
    surfacecontour = 0              #Linear
    use_texture = TRUE
    pattern = _("Clipboard Image")
    scale = 35.0
    tex_depth = -34.0
    invert = FALSE
    merge = FALSE
    pdb.python_layerfx_bevel_emboss(img, textLayer, 
                                    style, depth, direction, size, soften, angle, altitude, glosscontour, highlightcolor, highlightmode, 
                                    highlightopacity, shadowcolor, shadowmode, 
                                    shadowopacity, surfacecontour, use_texture, pattern, scale, tex_depth, invert, merge)
    
    #Text color overlay
    color = "#dfbc1a"
    opacity = 85.0
    mode = 16                       #LAYER_MODE_DODGE_LEGACY
    merge = FALSE
    pdb.python_layerfx_color_overlay(img, textLayer, color, opacity, mode, merge)

    #Text pattern overlay
    pattern = _("Clipboard Image")
    opacity = 100.0
    mode = 3                        #LAYER_MODE_MULTIPLY_LEGACY
    scale = 35
    interpolation_type = 0          #INTERPOLATION_NONE
    merge = FALSE
    pdb.python_layerfx_pattern_overlay(img, textLayer, pattern, opacity, mode, scale, interpolation_type, merge)
    
    #Create the floor reflection
    wh = img.width / 2
    hh = img.height / 2
    pdb.gimp_image_set_active_layer(img, inshadlayer)
    pdb.gimp_item_set_visible(wallLayer, FALSE)
    ReflectLayer = pdb.gimp_layer_new_from_visible(img, img, "Reflection")
    pdb.gimp_image_insert_layer(img, ReflectLayer, None, 0)
    ReflectLayer = pdb.gimp_item_transform_flip_simple(ReflectLayer, ORIENTATION_VERTICAL, TRUE, 0)
    pdb.gimp_drawable_offset(ReflectLayer, FALSE, OFFSET_TRANSPARENT, 0, 312)
    mask = pdb.gimp_layer_create_mask(ReflectLayer, ADD_MASK_BLACK)
    pdb.gimp_layer_add_mask(ReflectLayer, mask)
    pdb.gimp_context_set_default_colors()
    pdb.gimp_edit_blend(mask, 
                            0,  #blend-mode - blend_fg_bg_rgb
                            0,  #paint-mode - layer_mode_normal
                            0,  #gradient type - linear
                          100,  #opacity
                            0,  #offset
                            0,  #repeat
                        TRUE,  #reverse
                        FALSE,  #supersample
                            3,  #max-depth
                         0.20,  #threshold
                         TRUE,  #dither
                           wh,  #width1
                           hh,  #height1
                           wh,  #width2
               img.height +50)  #height2
    
    pdb.gimp_item_set_visible(wallLayer, TRUE)
    
    if mergeLayers==True:
        pdb.gimp_item_set_visible(wallLayer, FALSE)
        textLayer = pdb.gimp_image_merge_visible_layers(img, EXPAND_AS_NECESSARY)
        pdb.gimp_item_set_visible(wallLayer, TRUE)
        pdb.gimp_layer_resize_to_image_size(textLayer)
    
	#Resets the previous user defaults
    pdb.gimp_context_pop()	
	
register(
    "pm_create_a_honey_comb_text",
    "Creates a room background and a Bee honeycomb text",
    "Creates a room background and a Bee honeycomb text",
    "Pocholo",
    "Pocholo",
    "2022",
    "Create a Honeycomb text",
    "",
    [
    (PF_STRING, "customtext", "Text string", "HONEY"),
    (PF_FONT, "font", "Font face", "Vagabond"),
    (PF_TOGGLE, "mergeLayers", "Merge Layers?", 0),
    
    ],
    [], 
    pm_create_a_honey_comb_text, menu="<Image>/Pocholo's-Plugins",
    domain=("gimp20", gimp.locale_directory))
main()			   